@@ -1,4 +1,4 @@
-# Copyright (C) 1998-2009 by the Free Software Foundation, Inc.
+# Copyright (C) 1998-2015 by the Free Software Foundation, Inc.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -71,8 +71,16 @@ except NameError:
     True = 1
     False = 0
 
+try:
+    import dns.resolver
+    from dns.exception import DNSException
+    dns_resolver = True
+except ImportError:
+    dns_resolver = False
+
 EMPTYSTRING = ''
 UEMPTYSTRING = u''
+CR = '\r'
 NL = '\n'
 DOT = '.'
 IDENTCHARS = ascii_letters + digits + '_'
@@ -92,6 +100,12 @@ def list_exists(listname):
     #
     # The former two are for 2.1alpha3 and beyond, while the latter two are
     # for all earlier versions.
+    #
+    # But first ensure the list name doesn't contain a path traversal
+    # attack.
+    if len(re.sub(mm_cfg.ACCEPTABLE_LISTNAME_CHARACTERS, '', listname)) > 0:
+        syslog('mischief', 'Hostile listname: %s', listname)
+        return False
     basepath = Site.get_listpath(listname)
     for ext in ('.pck', '.pck.last', '.db', '.db.last'):
         dbfile = os.path.join(basepath, 'config' + ext)
@@ -212,17 +226,18 @@ def LCDomain(addr):
 
 
 # TBD: what other characters should be disallowed?
-_badchars = re.compile(r'[][()<>|;^,\000-\037\177-\377]')
-# characters in addition to _badchars which are not allowed in
-# unquoted local parts.
-_specials = re.compile(r'[:\\"]')
+_badchars = re.compile(r'[][()<>|:;^,\\"\000-\037\177-\377]')
+# Strictly speaking, some of the above are allowed in quoted local parts, but
+# this can open the door to certain web exploits so we don't allow them.
+# Only characters allowed in domain parts.
+_valid_domain = re.compile('[-a-z0-9]', re.IGNORECASE)
 
 def ValidateEmail(s):
     """Verify that an email address isn't grossly evil."""
     # Pretty minimal, cheesy check.  We could do better...
     if not s or s.count(' ') > 0:
         raise Errors.MMBadEmailError
-    if _badchars.search(s) or s[0] == '-':
+    if _badchars.search(s):
         raise Errors.MMHostileAddress, s
     user, domain_parts = ParseEmail(s)
     # This means local, unqualified addresses, are not allowed
@@ -230,10 +245,11 @@ def ValidateEmail(s):
         raise Errors.MMBadEmailError, s
     if len(domain_parts) < 2:
         raise Errors.MMBadEmailError, s
-    if not (user.startswith('"') and user.endswith('"')):
-        # local part is not quoted so it can't contain specials
-        if _specials.search(user):
-            raise Errors.MMBadEmailError, s
+    # domain parts may only contain ascii letters, digits and hyphen
+    # and must not begin with hyphen.
+    for p in domain_parts:
+        if len(p) == 0 or p[0] == '-' or len(_valid_domain.sub('', p)) > 0:
+            raise Errors.MMHostileAddress, s
 
 
 
@@ -251,6 +267,11 @@ def GetPathPieces(envar='PATH_INFO'):
     return None
 
 
+
+def GetRequestMethod():
+    return os.environ.get('REQUEST_METHOD')
+
+
 
 def ScriptURL(target, web_page_url=None, absolute=False):
     """target - scriptname only, nothing extra
@@ -426,6 +447,12 @@ def check_global_password(response, siteadmin=True):
 
 _ampre = re.compile('&amp;((?:#[0-9]+|[a-z]+);)', re.IGNORECASE)
 def websafe(s):
+    if mm_cfg.BROKEN_BROWSER_WORKAROUND:
+        # Archiver can pass unicode here. Just skip them as the
+        # archiver escapes non-ascii anyway.
+        if isinstance(s, str):
+            for k in mm_cfg.BROKEN_BROWSER_REPLACEMENTS:
+                s = s.replace(k, mm_cfg.BROKEN_BROWSER_REPLACEMENTS[k])
     # Don't double escape html entities
     return _ampre.sub(r'&\1', cgi.escape(s, quote=True))
 
@@ -593,7 +620,7 @@ ADMINDATA = {
     'set':         (3, 3),
     'subscribe':   (0, 3),
     'unsubscribe': (0, 1),
-    'who':         (0, 2),
+    'who':         (0, 1),
     }
 
 # Given a Message.Message object, test for administrivia (eg subscribe,
@@ -898,6 +925,61 @@ def oneline(s, cset):
         return EMPTYSTRING.join(s.splitlines())
 
 
+def strip_verbose_pattern(pattern):
+    # Remove white space and comments from a verbose pattern and return a
+    # non-verbose, equivalent pattern.  Replace CR and NL in the result
+    # with '\\r' and '\\n' respectively to avoid multi-line results.
+    if not isinstance(pattern, str):
+        return pattern
+    newpattern = ''
+    i = 0
+    inclass = False
+    skiptoeol = False
+    copynext = False
+    while i < len(pattern):
+        c = pattern[i]
+        if copynext:
+            if c == NL:
+                newpattern += '\\n'
+            elif c == CR:
+                newpattern += '\\r'
+            else:
+                newpattern += c
+            copynext = False
+        elif skiptoeol:
+            if c == NL:
+                skiptoeol = False
+        elif c == '#' and not inclass:
+            skiptoeol = True
+        elif c == '[' and not inclass:
+            inclass = True
+            newpattern += c
+            copynext = True
+        elif c == ']' and inclass:
+            inclass = False
+            newpattern += c
+        elif re.search('\s', c):
+            if inclass:
+                if c == NL:
+                    newpattern += '\\n'
+                elif c == CR:
+                    newpattern += '\\r'
+                else:
+                    newpattern += c
+        elif c == '\\' and not inclass:
+            newpattern += c
+            copynext = True
+        else:
+            if c == NL:
+                newpattern += '\\n'
+            elif c == CR:
+                newpattern += '\\r'
+            else:
+                newpattern += c
+        i += 1
+    return newpattern
+
+
 # Patterns and functions to flag possible XSS attacks in HTML.
 # This list is compiled from information at http://ha.ckers.org/xss.html,
 # http://www.quirksmode.org/js/events_compinfo.html,
@@ -911,99 +993,100 @@ _badwords = [
     # Kludge to allow the specific tag that's in the options.html template.
     '<link(?! rel="SHORTCUT ICON" href="<mm-favicon>">)',
     '<meta',
+    '<object',
     '<script',
-    r'(?:^|\W)j(?:ava)?script(?:\W|$)',
-    r'(?:^|\W)vbs(?:cript)?(?:\W|$)',
-    r'(?:^|\W)domactivate(?:\W|$)',
-    r'(?:^|\W)domattrmodified(?:\W|$)',
-    r'(?:^|\W)domcharacterdatamodified(?:\W|$)',
-    r'(?:^|\W)domfocus(?:in|out)(?:\W|$)',
-    r'(?:^|\W)dommenuitem(?:in)?active(?:\W|$)',
-    r'(?:^|\W)dommousescroll(?:\W|$)',
-    r'(?:^|\W)domnodeinserted(?:intodocument)?(?:\W|$)',
-    r'(?:^|\W)domnoderemoved(?:fromdocument)?(?:\W|$)',
-    r'(?:^|\W)domsubtreemodified(?:\W|$)',
-    r'(?:^|\W)fscommand(?:\W|$)',
-    r'(?:^|\W)onabort(?:\W|$)',
-    r'(?:^|\W)on(?:de)?activate(?:\W|$)',
-    r'(?:^|\W)on(?:after|before)print(?:\W|$)',
-    r'(?:^|\W)on(?:after|before)update(?:\W|$)',
-    r'(?:^|\W)onbefore(?:(?:de)?activate|copy|cut|editfocus|paste)(?:\W|$)',
-    r'(?:^|\W)onbeforeunload(?:\W|$)',
-    r'(?:^|\W)onbegin(?:\W|$)',
-    r'(?:^|\W)onblur(?:\W|$)',
-    r'(?:^|\W)onbounce(?:\W|$)',
-    r'(?:^|\W)onbroadcast(?:\W|$)',
-    r'(?:^|\W)on(?:cell)?change(?:\W|$)',
-    r'(?:^|\W)oncheckboxstatechange(?:\W|$)',
-    r'(?:^|\W)on(?:dbl)?click(?:\W|$)',
-    r'(?:^|\W)onclose(?:\W|$)',
-    r'(?:^|\W)oncommand(?:update)?(?:\W|$)',
-    r'(?:^|\W)oncomposition(?:end|start)(?:\W|$)',
-    r'(?:^|\W)oncontextmenu(?:\W|$)',
-    r'(?:^|\W)oncontrolselect(?:\W|$)',
-    r'(?:^|\W)oncopy(?:\W|$)',
-    r'(?:^|\W)oncut(?:\W|$)',
-    r'(?:^|\W)ondataavailable(?:\W|$)',
-    r'(?:^|\W)ondataset(?:changed|complete)(?:\W|$)',
-    r'(?:^|\W)ondrag(?:drop|end|enter|exit|gesture|leave|over)?(?:\W|$)',
-    r'(?:^|\W)ondragstart(?:\W|$)',
-    r'(?:^|\W)ondrop(?:\W|$)',
-    r'(?:^|\W)onend(?:\W|$)',
-    r'(?:^|\W)onerror(?:update)?(?:\W|$)',
-    r'(?:^|\W)onfilterchange(?:\W|$)',
-    r'(?:^|\W)onfinish(?:\W|$)',
-    r'(?:^|\W)onfocus(?:in|out)?(?:\W|$)',
-    r'(?:^|\W)onhelp(?:\W|$)',
-    r'(?:^|\W)oninput(?:\W|$)',
-    r'(?:^|\W)onkey(?:up|down|press)(?:\W|$)',
-    r'(?:^|\W)onlayoutcomplete(?:\W|$)',
-    r'(?:^|\W)on(?:un)?load(?:\W|$)',
-    r'(?:^|\W)onlosecapture(?:\W|$)',
-    r'(?:^|\W)onmedia(?:complete|error)(?:\W|$)',
-    r'(?:^|\W)onmouse(?:down|enter|leave|move|out|over|up|wheel)(?:\W|$)',
-    r'(?:^|\W)onmove(?:end|start)?(?:\W|$)',
-    r'(?:^|\W)on(?:off|on)line(?:\W|$)',
-    r'(?:^|\W)onoutofsync(?:\W|$)',
-    r'(?:^|\W)onoverflow(?:changed)?(?:\W|$)',
-    r'(?:^|\W)onpage(?:hide|show)(?:\W|$)',
-    r'(?:^|\W)onpaint(?:\W|$)',
-    r'(?:^|\W)onpaste(?:\W|$)',
-    r'(?:^|\W)onpause(?:\W|$)',
-    r'(?:^|\W)onpopup(?:hidden|hiding|showing|shown)(?:\W|$)',
-    r'(?:^|\W)onprogress(?:\W|$)',
-    r'(?:^|\W)onpropertychange(?:\W|$)',
-    r'(?:^|\W)onradiostatechange(?:\W|$)',
-    r'(?:^|\W)onreadystatechange(?:\W|$)',
-    r'(?:^|\W)onrepeat(?:\W|$)',
-    r'(?:^|\W)onreset(?:\W|$)',
-    r'(?:^|\W)onresize(?:end|start)?(?:\W|$)',
-    r'(?:^|\W)onresume(?:\W|$)',
-    r'(?:^|\W)onreverse(?:\W|$)',
-    r'(?:^|\W)onrow(?:delete|enter|exit|inserted)(?:\W|$)',
-    r'(?:^|\W)onrows(?:delete|enter|inserted)(?:\W|$)',
-    r'(?:^|\W)onscroll(?:\W|$)',
-    r'(?:^|\W)onseek(?:\W|$)',
-    r'(?:^|\W)onselect(?:start)?(?:\W|$)',
-    r'(?:^|\W)onselectionchange(?:\W|$)',
-    r'(?:^|\W)onstart(?:\W|$)',
-    r'(?:^|\W)onstop(?:\W|$)',
-    r'(?:^|\W)onsubmit(?:\W|$)',
-    r'(?:^|\W)onsync(?:from|to)preference(?:\W|$)',
-    r'(?:^|\W)onsyncrestored(?:\W|$)',
-    r'(?:^|\W)ontext(?:\W|$)',
-    r'(?:^|\W)ontimeerror(?:\W|$)',
-    r'(?:^|\W)ontrackchange(?:\W|$)',
-    r'(?:^|\W)onunderflow(?:\W|$)',
-    r'(?:^|\W)onurlflip(?:\W|$)',
-    r'(?:^|\W)seeksegmenttime(?:\W|$)',
-    r'(?:^|\W)svgabort(?:\W|$)',
-    r'(?:^|\W)svgerror(?:\W|$)',
-    r'(?:^|\W)svgload(?:\W|$)',
-    r'(?:^|\W)svgresize(?:\W|$)',
-    r'(?:^|\W)svgscroll(?:\W|$)',
-    r'(?:^|\W)svgunload(?:\W|$)',
-    r'(?:^|\W)svgzoom(?:\W|$)',
+    r'\bj(?:ava)?script\b',
+    r'\bvbs(?:cript)?\b',
+    r'\bdomactivate\b',
+    r'\bdomattrmodified\b',
+    r'\bdomcharacterdatamodified\b',
+    r'\bdomfocus(?:in|out)\b',
+    r'\bdommenuitem(?:in)?active\b',
+    r'\bdommousescroll\b',
+    r'\bdomnodeinserted(?:intodocument)?\b',
+    r'\bdomnoderemoved(?:fromdocument)?\b',
+    r'\bdomsubtreemodified\b',
+    r'\bfscommand\b',
+    r'\bonabort\b',
+    r'\bon(?:de)?activate\b',
+    r'\bon(?:after|before)print\b',
+    r'\bon(?:after|before)update\b',
+    r'\bonbefore(?:(?:de)?activate|copy|cut|editfocus|paste)\b',
+    r'\bonbeforeunload\b',
+    r'\bonbegin\b',
+    r'\bonblur\b',
+    r'\bonbounce\b',
+    r'\bonbroadcast\b',
+    r'\bon(?:cell)?change\b',
+    r'\boncheckboxstatechange\b',
+    r'\bon(?:dbl)?click\b',
+    r'\bonclose\b',
+    r'\boncommand(?:update)?\b',
+    r'\boncomposition(?:end|start)\b',
+    r'\boncontextmenu\b',
+    r'\boncontrolselect\b',
+    r'\boncopy\b',
+    r'\boncut\b',
+    r'\bondataavailable\b',
+    r'\bondataset(?:changed|complete)\b',
+    r'\bondrag(?:drop|end|enter|exit|gesture|leave|over)?\b',
+    r'\bondragstart\b',
+    r'\bondrop\b',
+    r'\bonend\b',
+    r'\bonerror(?:update)?\b',
+    r'\bonfilterchange\b',
+    r'\bonfinish\b',
+    r'\bonfocus(?:in|out)?\b',
+    r'\bonhelp\b',
+    r'\boninput\b',
+    r'\bonkey(?:up|down|press)\b',
+    r'\bonlayoutcomplete\b',
+    r'\bon(?:un)?load\b',
+    r'\bonlosecapture\b',
+    r'\bonmedia(?:complete|error)\b',
+    r'\bonmouse(?:down|enter|leave|move|out|over|up|wheel)\b',
+    r'\bonmove(?:end|start)?\b',
+    r'\bon(?:off|on)line\b',
+    r'\bonoutofsync\b',
+    r'\bonoverflow(?:changed)?\b',
+    r'\bonpage(?:hide|show)\b',
+    r'\bonpaint\b',
+    r'\bonpaste\b',
+    r'\bonpause\b',
+    r'\bonpopup(?:hidden|hiding|showing|shown)\b',
+    r'\bonprogress\b',
+    r'\bonpropertychange\b',
+    r'\bonradiostatechange\b',
+    r'\bonreadystatechange\b',
+    r'\bonrepeat\b',
+    r'\bonreset\b',
+    r'\bonresize(?:end|start)?\b',
+    r'\bonresume\b',
+    r'\bonreverse\b',
+    r'\bonrow(?:delete|enter|exit|inserted)\b',
+    r'\bonrows(?:delete|enter|inserted)\b',
+    r'\bonscroll\b',
+    r'\bonseek\b',
+    r'\bonselect(?:start)?\b',
+    r'\bonselectionchange\b',
+    r'\bonstart\b',
+    r'\bonstop\b',
+    r'\bonsubmit\b',
+    r'\bonsync(?:from|to)preference\b',
+    r'\bonsyncrestored\b',
+    r'\bontext\b',
+    r'\bontimeerror\b',
+    r'\bontrackchange\b',
+    r'\bonunderflow\b',
+    r'\bonurlflip\b',
+    r'\bseeksegmenttime\b',
+    r'\bsvgabort\b',
+    r'\bsvgerror\b',
+    r'\bsvgload\b',
+    r'\bsvgresize\b',
+    r'\bsvgscroll\b',
+    r'\bsvgunload\b',
+    r'\bsvgzoom\b',
     ]
 
 
@@ -1049,3 +1132,127 @@ def suspiciousHTML(html):
     else:
         return False
 
+
+# This takes an email address, and returns True if DMARC policy is p=reject
+# or possibly quarantine.
+def IsDMARCProhibited(mlist, email):
+    if not dns_resolver:
+        # This is a problem; log it.
+        syslog('error',
+            'DNS lookup for dmarc_moderation_action for list %s not available',
+            mlist.real_name)
+        return False
+
+    email = email.lower()
+    at_sign = email.find('@')
+    if at_sign < 1:
+        return False
+    dmarc_domain = '_dmarc.' + email[at_sign+1:]
+
+    try:
+        resolver = dns.resolver.Resolver()
+        resolver.timeout = float(mm_cfg.DMARC_RESOLVER_TIMEOUT)
+        resolver.lifetime = float(mm_cfg.DMARC_RESOLVER_LIFETIME)
+        txt_recs = resolver.query(dmarc_domain, dns.rdatatype.TXT)
+    except (dns.resolver.NXDOMAIN, dns.resolver.NoAnswer):
+        return False
+    except DNSException, e:
+        syslog('error',
+               'DNSException: Unable to query DMARC policy for %s (%s). %s',
+              email, dmarc_domain, e.__class__)
+        return False
+    else:
+# people are already being dumb, don't trust them to provide honest DNS
+# where the answer section only contains what was asked for, nor to include
+# CNAMEs before the values they point to.
+        full_record = ""
+        results_by_name = {}
+        cnames = {}
+        want_names = set([dmarc_domain + '.'])
+        for txt_rec in txt_recs.response.answer:
+            if txt_rec.rdtype == dns.rdatatype.CNAME:
+                cnames[txt_rec.name.to_text()] = (
+                    txt_rec.items[0].target.to_text())
+            if txt_rec.rdtype != dns.rdatatype.TXT:
+                continue
+            results_by_name.setdefault(txt_rec.name.to_text(), []).append(
+                "".join(txt_rec.items[0].strings))
+        expands = list(want_names)
+        seen = set(expands)
+        while expands:
+            item = expands.pop(0)
+            if item in cnames:
+                if cnames[item] in seen:
+                    continue # cname loop
+                expands.append(cnames[item])
+                seen.add(cnames[item])
+                want_names.add(cnames[item])
+                want_names.discard(item)
+
+        if len(want_names) != 1:
+            syslog('error',
+                   """multiple DMARC entries in results for %s,
+                   processing each to be strict""",
+                   dmarc_domain)
+        for name in want_names:
+            if name not in results_by_name:
+                continue
+            dmarcs = filter(lambda n: n.startswith('v=DMARC1;'),
+                            results_by_name[name])
+            if len(dmarcs) == 0:
+                return False
+            if len(dmarcs) > 1:
+                syslog('error',
+                       """RRset of TXT records for %s has %d v=DMARC1 entries;
+                       testing them all""",
+                        dmarc_domain, len(dmarc))
+            for entry in dmarcs:
+                if re.search(r'\bp=reject\b', entry, re.IGNORECASE):
+                    syslog('vette',
+                        'DMARC lookup for %s (%s) found p=reject in %s = %s',
+                        email, dmarc_domain, name, entry)
+                    return True
+
+                if (mlist.dmarc_quarantine_moderation_action and
+                    re.search(r'\bp=quarantine\b', entry, re.IGNORECASE)):
+                    syslog('vette',
+                      'DMARC lookup for %s (%s) found p=quarantine in %s = %s',
+                            email, dmarc_domain, name, entry)
+                    return True
+
+    return False
+
+
+def check_eq_domains(email, domains_list):
+    """The arguments are an email address and a string representing a
+    list of lists in a form like 'a,b,c;1,2' representing [['a', 'b',
+    'c'],['1', '2']].  The inner lists are domains which are
+    equivalent in some sense.  The return is an empty list or a list
+    of email addresses equivalent to the first argument.
+    For example, given
+
+    email = 'user@me.com'
+    domains_list = '''domain1, domain2; mac.com, me.com, icloud.com;
+                   domaina, domainb
+                   '''
+
+    check_eq_domains(email, domains_list) will return
+    ['user@mac.com', 'user@icloud.com']
+    """
+    if not domains_list:
+        return []
+    try:
+        local, domain = email.rsplit('@', 1)
+    except ValueError:
+        return []
+    domain = domain.lower()
+    domains_list = re.sub('\s', '', domains_list).lower()
+    domains = domains_list.split(';')
+    domains_list = []
+    for d in domains:
+        domains_list.append(d.split(','))
+    for domains in domains_list:
+        if domain in domains:
+            return [local + '@' + x for x in domains if x != domain]
+    return []
+
