@@ -1,4 +1,4 @@
-# LaTeX2HTML 2002-2-1 (1.71)
+# LaTeX2HTML 2008 (1.71)
 # Associate internals original text with physical files.
 
 
@@ -10,6 +10,14 @@ $key = q/contents/;
 $ref_files{$key} = "$dir".q|node2.html|; 
 $noresave{$key} = "$nosave";
 
+$key = q/sender-filters/;
+$ref_files{$key} = "$dir".q|node22.html|; 
+$noresave{$key} = "$nosave";
+
+$key = q/general-personality/;
+$ref_files{$key} = "$dir".q|node10.html|; 
+$noresave{$key} = "$nosave";
+
 $key = q/about/;
 $ref_files{$key} = "$dir".q|node36.html|; 
 $noresave{$key} = "$nosave";
